<?php
/**
 * Juno_Driver:: defines an API for implementing storage backends for
 * Juno.
 *
 * $Horde: juno/lib/Driver.php,v 1.14 2004/09/12 22:43:52 mdjukic Exp $
 *
 * @author  Mathieu Legare <legare@uqtr.ca>
 * @version $Revision: 1.14 $
 * @since   Juno 0.3
 * @package Juno
 */
class Juno_Driver {

    /**
     * Hash containing connection parameters.
     *
     * @var array $_params
     */
    var $_params = array();

    /**
     * Constructs a new Juno driver object.
     *
     * @param array $params  A hash containing connection parameters.
     */
    function Juno_Driver($params = array())
    {
        $this->_params = $params;
    }

    /**
     * Attempts to return a concrete Juno_Driver instance based on
     * $driver.
     *
     * @param string    $driver     The type of concrete Juno_Driver subclass
     *                              to return. The is based on the storage
     *                              driver ($driver). The code is dynamically
     *                              included.
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The newly created concrete Juno_Driver instance, or
     *                  false on an error.
     */
    function &factory($driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        $driver = basename($driver);

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        include_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'Juno_Driver_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($params);
        } else {
            Horde::fatal(PEAR::raiseError(sprintf(_("No such backend '%s' found"), $driver)), __FILE__, __LINE__);
        }
    }

    /**
     * Attempts to return a reference to a concrete Juno_Driver instance
     * based on $driver.  It wil only create a new instance if no
     * Juno_Driver instance with the same parameters currently exists.
     *
     * This should be used if multiple storage sources are required.
     *
     * This method must be invoked as: $var = &Juno_Driver::singleton()
     *
     * @param string    $driver     The type of concrete Juno_Driver subclass
     *                              to return.  The is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The created concrete Juno_Driver instance, or false
     *                  on error.
     */
    function &singleton($driver = null, $params = null)
    {
        static $instances;

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Juno_Driver::factory($driver, $params);
        }

        return $instances[$signature];
    }

}
