<?php
/**
 * $Horde: juno/lib/Juno.php,v 1.27 2005/01/03 14:35:10 jan Exp $
 *
 * Copyright 2002-2005 Mathieu Legare <legare@uqtr.ca>
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */
class Juno {

    function Juno()
    {
        /* Category driver for books. */
        $GLOBALS['books'] = &new Juno_Books;

        /* Set up the book object and status display. */
        $GLOBALS['book'] = $GLOBALS['books']->getCurrentBook();
        $GLOBALS['book_status'] = Juno::getBookStatus($GLOBALS['book']);
    }

    function _readURL($url)
    {
        $options['method'] = 'GET';
        $options['timeout'] = 5;
        $options['allowRedirects'] = true;

        require_once 'HTTP/Request.php';
        $http = &new HTTP_Request($url, $options);
        @$http->sendRequest();
        if ($http->getResponseCode() != 200) {
            return PEAR::raiseError(sprintf(_("Could not open %s."), $url));
        }

        return $http->getResponseBody();
    }

    /**
     * @static
     */
    function prefEncode($pref, $val)
    {
        require_once 'Horde/Serialize.php';
        $val = Horde_Serialize::serialize($val, SERIALIZE_UTF7_BASIC);
        return $GLOBALS['prefs']->setValue($pref, $val);
    }

    /**
     * @static
     */
    function prefDecode($pref)
    {
        require_once 'Horde/Serialize.php';
        $val = $GLOBALS['prefs']->getValue($pref);
        return Horde_Serialize::unserialize($val, SERIALIZE_UTF7_BASIC);
    }

    /**
     * Build Juno's list of menu items.
     *
     * @access public
     */
    function getMenu($returnType = 'object')
    {
        require_once 'Horde/Menu.php';

        $menu = &new Menu();

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

    function isBookOpen($redirect = true)
    {
        if (empty($GLOBALS['book'])) {
            if ($redirect) {
                $GLOBALS['notification']->push(_("Open a book first."), 'horde.warning');
                $url = Horde::applicationUrl('books/index.php', true);
                header('Location: ' . $url);
                exit;

            }
            return false;
        }

        return true;
    }

    function getBookStatus(&$book)
    {
        $book_status = array();
        if (!is_a($book, 'Book_DataTreeObject')) {
            return array();
        }

        $book_status['name'] = $book->getName();
        $book_status['icon'] = Horde::img('book_open.png', _("Close book"), 'align="middle"');
        $url = Util::addParameter(Horde::applicationUrl('books/index.php'), 'close', true);
        $book_status['close_link'] = Horde::link($url, _("Close book"), '', '', '', _("Close book"));

        return $book_status;
    }

    function getTabs($view)
    {
        $tab_list = array();
        switch ($view) {
        case 'lists':
            $tab_list = array('accounts' => _("Accounts"),
                              'currencies' => _("Currencies"),
                              'items' => _("Items"),
                              'expenses' => _("Expenses"));
            break;
        }

        foreach ($tab_list as $tab_id => $tab) {
            $url = Horde::applicationUrl($view . '/' . $tab_id . '.php');
            $tabs[] = array('link' => Horde::link($url, $tab, '', '', '', $tab),
                            'text' => $tab);
        }

        return $tabs;
    }

    function getAccountClasses()
    {
        return array('asset' => _("Asset"),
                     'liability' => _("Liability"),
                     'income' => _("Income"),
                     'expense' => _("Expense"),
                     'equity' => _("Equity"),
                     'costofsales' => _("Cost of Sales"),
                     'otherincome' => _("Other Income"),
                     'otherexpense' => _("Other Expense"));
    }

    function getAccountTypes()
    {
        return array('header' => _("Header"),
                     'detail' => _("Detail Account"),
                     'checking' => _("Detail Checking Account"),
                     'creditcard' => _("Detail Credit Card Account"));
    }

    /**
     * Returns the column to sort by, checking first if it is specified in the
     * URL, then returning the value stored in prefs.
     *
     * @access public
     *
     * @param string $view  The view name, used to identify preference settings
     *                      for sorting.
     *
     * @return string  The column to sort by.
     */
    function getSortBy($view)
    {
        global $prefs;
        if (($sortby = Util::getFormData($view . '_sortby')) !== null) {
            $prefs->setValue($view . '_sortby', $sortby);
        }
        return $prefs->getValue($view . '_sortby');
    }

    /**
     * Returns the sort direction, checking first if it is specified in the URL,
     * then returning the value stored in prefs.
     *
     * @access public
     *
     * @param string $view  The view name, used to identify preference settings
     *                      for sorting.
     *
     * @return integer  The sort direction, 0 = ascending, 1 = descending.
     */
    function getSortDir($view)
    {
        global $prefs;
        if (($sortdir = Util::getFormData($view . '_sortdir')) !== null) {
            $prefs->setValue($view . '_sortdir', $sortdir);
        }
        return $prefs->getValue($view . '_sortdir');
    }

    /**
     * Formats column headers have sort links and sort arrows.
     *
     * @access public
     *
     * @param array  $columns   The columns to format.
     * @param string $sort_by   The current 'sort-by' column.
     * @param string $sort_dir  The current sort direction.
     * @param string $view      The view name, used to identify preference
     *                          settings for sorting.
     *
     * @return array  The formated column headers to be displayed.
     */
    function formatColumnHeaders($columns, $sort_by, $sort_dir, $view)
    {
        /* Get the current url, remove any sorting parameters. */
        $url = Horde::selfURL(true);
        $url = Util::removeParameter($url, array($view . '_sortby', $view . '_sortdir'));

        /* Go through the column headers to format and add sorting links. */
        foreach ($columns as $col_name => $col_title) {
            if ($sort_by == $col_name) {
                /* This column is currently sorted by, plain title and add sort
                   direction arrow. */
                $sort_img = ($sort_dir ? 'za.png' : 'az.png');
                $sort_title = ($sort_dir ? _("Sort Ascending") : _("Sort Descending"));
                $col_arrow = Horde::link(Util::addParameter($url, array($view . '_sortby' => $col_name, $view . '_sortdir' => $sort_dir ? 0 : 1))) . Horde::img($sort_img, $sort_title, null, $GLOBALS['registry']->getImageDir('horde')) . '</a>';
            } else {
                /* Column not currently sorted, add link to sort by this one and
                   no sort arrow. */
                $col_arrow = '';
                $col_title = Horde::link(Util::addParameter($url, $view . '_sortby', $col_name)) . $col_title . '</a>';
            }
            $columns[$col_name] = $col_arrow . $col_title;
        }

        return $columns;
    }

    /**
     * Set the standard fields for transaction input.
     */
    function setTransactionFields(&$form)
    {
        global $book;

        $form->addHidden('', 'transaction_id', 'int', false);
        $form->addVariable(_("Transaction details"), 'trans_details', 'header', false);
        $v = &$form->addVariable(_("Enter date"), 'enterdate', 'monthdayyear', true, false, null, array(date('Y'), date('Y') - 5, true, 2));
        $v->setDefault(Juno::getLastDate());
        $v = &$form->addVariable(_("Post date"), 'postdate', 'monthdayyear', true, false, null, array(date('Y'), date('Y') - 5, true, 2));
        $v->setDefault(Juno::getLastDate());
        $form->addVariable(_("Reference number"), 'ref', 'text', true);
        if ($book->has('multicurrency')) {
            $v = &$form->addVariable(_("Currency"), 'currency', 'enum', true, false, null, array(Juno::getCurrencies(), true));
            $v->setDefault($book->getCurrency());
        }
        $form->addVariable(_("Description"), 'desc', 'text', false);
    }

    /**
     * If the last used date has been saved to the session return that,
     * otherwise return the current date.
     *
     * @access public
     *
     * @return string  The date in YYYY-MM-DD format.
     */
    function getLastDate()
    {
        return isset($_SESSION['juno']['last_date']) ?
                    $_SESSION['juno']['last_date'] : date('Y-m-d');
    }

    /**
     * Returns an array of available currencies.
     *
     * @access public
     *
     * @return array  Available currencies.
     * @todo  Move this to NLS?
     */
    function getCurrencies()
    {
        return array('EUR' => 'Euro',
                     'USD' => 'US Dollars');
    }

    /**
     * Returns an array of available invoice terms.
     *
     * @access public
     *
     * @return array  Available terms.
     * @todo  Create a more dynamic way of setting these up?
     */
    function getTerms()
    {
        $terms = array('netmonth' => _("Net monthly"),
                       'net7'     => _("7 days net"),
                       'net30'    => _("30 days net"),
                       'net60'    => _("60 days net"),
                       'net90'    => _("90 days net"),
                       'EOM'      => _("End of month"),
                       'COD'      => _("Cash on delivery"),
                       'CBS'      => _("Cash before shipment"),
                       'custom'   => _("Custom date"));
        return $terms;
    }

}
