<?php
/**
 * The Juno_Lists class provides the common functions shared between the
 * various types of lists present in Juno. The factory() function sets up the
 * rest of the functions specific to the type of list.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: juno/lib/Lists.php,v 1.12 2005/01/03 14:35:10 jan Exp $
 *
 * @author Marko Djukic <marko@oblo.com>
 * @package Juno
 * @todo Add the classes for the other types of lists - expenses/contacts/items.
 */
class Juno_Lists {

    /* DataTree object. */
    var $_list;

    /* The list type. */
    var $_type;

    /* Stubs. */
    function _saveData() { }


    function getList($list_root = DATATREE_ROOT, $formatted = true, $no_root = false, $root_name = null)
    {
        if (is_null($root_name)) {
            $root_name = _("-- Top Level --");
        }
        $list = $this->_list->get(DATATREE_FORMAT_FLAT, $list_root, true, $root_name);

        /* If root is not required, unset it out of the array. */
        if ($no_root) {
            unset($list[$list_root]);
        }

        if (!$formatted) {
            return $list;
        }

        return $list;
    }

    function saveListEntry(&$info, $path_prefix = '')
    {
        if (empty($info['id'])) {
            if (!empty($path_prefix)) {
                $path_prefix .= ':';
            }
            /* If root, path is blank. */
            $path = ($info['parent_id'] != DATATREE_ROOT ? $this->_list->getName($info['parent_id']) . ':' : $path_prefix);
            $path .= $info['name'];
            $list_entry = &$this->newListEntry($path);
        } else {
            $list_entry = $this->get($info['id']);
        }

        /* Add the data to the category object. */
        $this->_saveData($list_entry, $info);

        if (empty($info['id'])) {
            /* New list entry, add the category object. */
            $result = $this->_list->add($list_entry);
            $info['id'] = $this->_list->getId($list_entry);
        } else {
            /* Existing list entry, first save the category data. */
            $this->_list->updateData($list_entry);
        }

        return $info['id'];
    }

    function &newListEntry($name)
    {
        if (empty($name)) {
            return PEAR::raiseError(_("Can not create a list entry with an empty name."));
        }
        $class = 'List_' . $this->_type . '_DataTreeObject';
        $list_entry = &new $class($name);
        $list_entry->setListsOb($this);
        return $list_entry;
    }

    function getParentId($child)
    {
        return $this->_list->getParent($child);
    }

    function &factory($type)
    {
        @require_once dirname(__FILE__) . '/Lists/' . $type . '.php';

        /* Connect to the backend for lists. */
        $class = 'Juno_Lists_' . $type;
        if (class_exists($class)) {
            return $ret = &new $class($type);
        } else {
            return PEAR::raiseError(sprintf(_("Class definition of '%s' not found for Juno_Lists"), $class));
        }

    }

    /**
     * Attempts to return a reference to a concrete Juno_Lists instance. It will
     * only create a new instance if no Juno_Lists instance currently exists.
     *
     * This method must be invoked as: $var = &Juno_Lists::singleton($type)
     *
     * @param string $type  The type of list to set up.
     *
     * @return object Juno_Lists  The concrete Juno_Lists reference, or false on
     *                            an error.
     */
    function &singleton($type)
    {
        static $lists;

        if (!isset($lists[$type])) {
            $lists[$type] = &Juno_Lists::factory($type);
        }

        return $lists[$type];
    }

}

class List_DataTreeObject extends DataTreeObject {

    var $_listsOb;

    function List_DataTreeObject($name)
    {
        parent::DataTreeObject($name);
    }

    function setListsOb(&$listsOb)
    {
        $this->_listsOb = &$listsOb;
    }

    function saveListData()
    {
        return $this->_listsOb->_list->updateData($this);
    }

}
