<?php
/**
 * The Juno_Lists_accounts class extends the main Juno_Lists class with
 * functionality specific to the list of accounts and uses the Horde_DataTree
 * as its backend storage.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: juno/lib/Lists/accounts.php,v 1.10 2005/01/03 14:35:10 jan Exp $
 *
 * @author Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.10 $
 * @package Juno
 * @todo Add the full set of functions for handling accounts in Horde_DataTree.
 */
class Juno_Lists_accounts extends Juno_Lists {

    /**
     * Constructor
     */
    function Juno_Lists_accounts($type)
    {
        global $conf;

        if (empty($conf['datatree']['driver'])) {
            Horde::fatal(_("You must configure a Horde DataTree backend to use Juno."), __FILE__, __LINE__);
        }
        $driver = $conf['datatree']['driver'];
        $group = 'juno.lists.accounts.' . $GLOBALS['book']->getId();
        $params = array_merge(Horde::getDriverConfig('datatree', $driver), array('group' => $group));

        /* Connect to the backend for accounts lists. */
        $this->_list = &DataTree::singleton($driver, $params);

        /* Set the current list type. */
        $this->_type = $type;
    }

    function _formatList(&$list)
    {
    }

    function _saveData(&$account, &$info)
    {
        $account->data['class'] = $info['class'];
        $account->data['type'] = $info['type'];
        $account->data['number'] = $info['number'];
        $account->data['name'] = $info['name'];
        $account->data['description'] = $info['description'];
        $account->data['start_balance'] = $info['start_balance'];
    }

}

class List_Accounts_DataTreeObject extends List_DataTreeObject {

    function _toAttributes()
    {
        $attributes = array();
        $attributes[] = array('name'  => 'a',
                              'key'   => 'class',
                              'value' => $this->data['class']);
        $attributes[] = array('name'  => 'a',
                              'key'   => 'type',
                              'value' => $this->data['type']);
        $attributes[] = array('name'  => 'a',
                              'key'   => 'number',
                              'value' => $this->data['number']);
        $attributes[] = array('name'  => 'a',
                              'key'   => 'description',
                              'value' => $this->data['description']);
        $attributes[] = array('name'  => 'a',
                              'key'   => 'start_balance',
                              'value' => $this->data['start_balance']);
        return $attributes;
    }

    function _fromAttributes($attributes)
    {
        $this->data = array();

        foreach ($attributes as $attr) {
            if (!isset($this->data[$attr['name']])) {
                $this->data[$attr['name']] = array();
            }
            $this->data[$attr['name']][$attr['key']] = $attr['value'];
        }
    }

}
