<?php
/**
 * Juno_Stocks
 *
 * Copyright (c) 2000-2004 Charles Hagenbuch <chuck@horde.org>.
 *
 * $Horde: juno/lib/Stocks.php,v 1.19 2004/10/19 15:21:59 mdjukic Exp $
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.19 $
 * @package Juno
 */
class Juno_Stocks {

    /**
     * Array of capabilities that this backend supports.
     * @var array $_capabilities
     */
    var $_capabilities = array();

    /**
     * Gets an array containing the user's currently selected ticker
     * symbols.
     *
     * @return array  The array of stock ticker symbols.
     */
    function getTickerSymbols()
    {
        global $prefs;

        $symbols = array();
        $portfolio = Juno::prefDecode('portfolio');
        if (is_array($portfolio)) {
            foreach ($portfolio as $info) {
                $symbols[] = trim($info[0]);
            }
        }

        if ($prefs->getValue('djia')) {
            array_unshift($symbols, '^DJI');
        }
        if ($prefs->getValue('sp500')) {
            array_unshift($symbols, '^GSPC');
        }
        if ($prefs->getValue('nasdaq')) {
            array_unshift($symbols, '^IXIC');
        }

        return $symbols;
    }

    /**
     * Gets a listing of stock quotes for the specified ticker
     * symbols.
     *
     * @param array $symbols  List of stock/fund/index symbols to display.
     *
     * @return string  HTML representation of stock quotes.
     */
    function getStockListing($symbols)
    {
        $html = '<table border="0" cellpadding="0" cellspacing="1" width="100%" class="item">';
        $html .= '<tr class="item"><td width="1%"><b>' . _("Chart") . '</b></td><td><b>' . _("Symbol") . '</b></td><td><b>' . _("Name") . '</b></td><td><b>' . _("Price") . '</b></td><td><b>' . _("Change") . '</b></td><td><b>' . _("Value") . '</b></td></tr>';

        $stocks = &Juno_Stocks::singleton('yahoo');
        $quotes = $stocks->getQuotes($symbols);

        /* Get portfolio so we can compute the value of any stocks we
         * have a number of shares for. */
        $portfolio = Juno::prefDecode('portfolio');
        $counts = array();
        foreach ($portfolio as $info) {
            if (!empty($info[1]) && $info[1] != '*') {
                $counts[String::lower($info[0])] = $info[1];
            }
        }

        $portfolioValue = 0;
        $portfolioDelta = 0;
        foreach ($quotes as $symbol => $quote) {
            $html .= '<tr class="text"><td align="center">';
            $html .= Horde::link(Util::addParameter(Horde::applicationUrl('stocks/chart.php'), 'ticker', $symbol),
                                 _("View Chart"), '', '',
                                 "window.open('" . Util::addParameter(Horde::applicationUrl('stocks/chart.php', true), 'ticker', $symbol) .
                                 "', '_blank', 'toolbar=no,menubar=no,status=no,resizable=yes,location=no,scrollbars=yes,width=220,height=120'); return false;");
            $html .= Horde::img('stocks.png', _("View Chart")) . '</a></td>';

            $html .= '<td nowrap="nowrap">';
            $html .= Horde::link(Util::addParameter(Horde::applicationUrl('stocks/quote.php'), 'ticker', $symbol),
                                 _("View Quote Details"), '', '',
                                 "window.open('" . Util::addParameter(Horde::applicationUrl('stocks/quote.php', true), 'ticker', $symbol) .
                                 "', '_blank', 'toolbar=no,menubar=no,status=no,resizable=yes,location=no,scrollbars=yes,width=340,height=370'); return false;");
            $html .= htmlspecialchars(String::upper($symbol)) . '</a></td>';
            $html .= '<td>' . (isset($quote['name']) ? htmlspecialchars($quote['name']) : '') . '</td><td>';
            if (isset($quote['lastPrice'])) {
                $html .= $quote['lastPrice'];
            }
            $html .= '</td><td>';
            if (!empty($quote['changePrice'])) {
                if ($quote['changePrice'] > 0) {
                    $html .= '<span style="color:green">';
                } elseif ($quote['changePrice'] < 0) {
                    $html .= '<span style="color:red">';
                }
                $html .= $quote['changePrice'];
                if ($quote['changePrice'] != 0) {
                    $html .= '</span>';
                }
            }
            $html .= '</td><td>';
            if (!empty($counts[$symbol]) && !empty($quote['lastPrice'])) {
                $html .= '$' . number_format($counts[$symbol] * $quote['lastPrice'], 2);
                $portfolioValue += ($counts[$symbol] * $quote['lastPrice']);
                $portfolioDelta += ($counts[$symbol] * $quote['changePrice']);
            }
            $html .= '</td></tr>';
        }

        /**
         * Print total value and delta.
         */
        $html .= '<tr class="text"><td>&nbsp;</td><td>&nbsp;</td><td><b>';
        $html .= _("Portfolio Total");
        $html .= '</b></td><td>&nbsp;</td><td><b>';
        if ($portfolioDelta > 0) {
            $html .= '<span style="color:green">+';
        } elseif ($portfolioDelta < 0) {
            $html .= '<span style="color:red">';
        }
        $html .= number_format($portfolioDelta, 2) . '</b></td><td><b>';
        if ($portfolioDelta != 0) {
            $html .= '</span>';
        }
        $html .= '$' . number_format($portfolioValue, 2);
        $html .= '</b></td></tr>';

        $html .= '</table>';
        return $html;
    }

    /**
     * See if the current backend supports an action.
     *
     * @param string $capability  The capability to check.
     *
     * @return boolean  Whether or not the requested action is supported.
     */
    function hasCapability($capability)
    {
        return !empty($this->_capabilities[$capability]);
    }

    /**
     * Fetch a set of stock information, updating any that are out of
     * date.
     */
    function getQuotes($symbols)
    {
        global $conf;

        if (!is_array($symbols)) {
            $symbols = explode(',', $symbols);
        }

        require_once 'Horde/Cache.php';
        $cache = &Horde_Cache::singleton($conf['cache']['driver'], Horde::getDriverConfig('cache', $conf['cache']['driver']));
        if (is_a($cache, 'PEAR_Error')) {
            Horde::fatal($cache, __FILE__, __LINE__);
        }

        $stocks = array();
        $needToUpdate = array();
        foreach ($symbols as $symbol) {
            $symbol = String::lower($symbol);
            $stocks[$symbol] = @unserialize($cache->get('juno.stock.' . $symbol, $GLOBALS['conf']['stocks']['expire']));
            if (!$stocks[$symbol]) {
                $needToUpdate[] = $symbol;
            }
        }

        if (count($needToUpdate)) {
            $updatedStocks = $this->_getQuotes($needToUpdate);
            if (!is_a($updatedStocks, 'PEAR_Error')) {
                foreach ($updatedStocks as $symbol => $data) {
                    $stocks[$symbol] = $data;
                    $cache->set('juno.stock.' . $symbol, serialize($data));
                }
            }
        }

        return $stocks;
    }

    function getChart($symbol, $type = 't', $modifier = '')
    {
        global $conf;

        if (!$this->hasCapability('charts')) {
            return false;
        }

        require_once 'Horde/Cache.php';
        $cache = &Horde_Cache::singleton($conf['cache']['driver'], Horde::getDriverConfig('cache', $conf['cache']['driver']));
        if (is_a($cache, 'PEAR_Error')) {
            Horde::fatal($cache, __FILE__, __LINE__);
        }

        $key = 'juno.stock.chart.' . serialize(array($symbol, $type, $modifier));
        $chart = $cache->get($key, $GLOBALS['conf']['stocks']['expire']);
        if (!$chart) {
            $chart = $this->_getChart($symbol, $type, $modifier);
            $cache->set($key, $chart);
        }

        return $chart;
    }

    /**
     * Attempts to return a concrete Juno_Stocks instance based on $driver.
     *
     * @param string    $driver   The type of concrete Juno_Stocks subclass
     *                            to return. The is based on the storage
     *                            driver ($driver). The code is dynamically
     *                            included.
     *
     * @param array     $params   (optional) A hash containing any additional
     *                            configuration or connection parameters a
     *                            subclass might need.
     *
     * @return mixed    The newly created concrete Juno_Stocks instance, or
     *                  false on an error.
     */
    function &factory($driver, $params = array())
    {
        $driver = basename($driver);
        include_once dirname(__FILE__) . '/Stocks/' . $driver . '.php';
        $class = 'Juno_Stocks_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($params);
        } else {
            Horde::fatal(PEAR::raiseError(sprintf(_("No such backend '%s' found"), $driver)), __FILE__, __LINE__);
        }
    }

    /**
     * Attempts to return a reference to a concrete Juno_Stocks instance
     * based on $driver.  It wil only create a new instance if no
     * Juno_Stocks instance with the same parameters currently exists.
     *
     * This should be used if multiple storage sources are required.
     *
     * This method must be invoked as: $var = &Juno_Stocks::singleton()
     *
     * @param string    $driver   The type of concrete Juno_Stocks subclass
     *                            to return. The is based on the storage
     *                            driver ($driver). The code is dynamically
     *                            included.
     *
     * @param array     $params   (optional) A hash containing any additional
     *                            configuration or connection parameters a
     *                            subclass might need.
     *
     * @return mixed    The created concrete Juno_Stocks instance, or false
     *                  on error.
     */
    function &singleton($driver = null, $params = array())
    {
        static $instances;

        if (!isset($instances)) {
            $instances = array();
        }

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['stocks']['driver'];
            $params = Horde::getDriverConfig('stocks', $driver);
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Juno_Stocks::factory($driver, $params);
        }

        return $instances[$signature];
    }

}
