<?php
/**
 * Juno_Stocks_yahoo
 * 
 * Copyright (c) 2003 Charles Hagenbuch <chuck@horde.org>.
 *
 * The information that you obtain with this library may be
 * copyrighted by Yahoo! Inc., and is governed by their usage license.
 * See http://www.yahoo.com/docs/info/gen_disclaimer.html for more
 * information.
 *
 * $Horde: juno/lib/Stocks/yahoo.php,v 1.14 2004/10/19 15:21:59 mdjukic Exp $
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.14 $
 * @package Juno
 */
class Juno_Stocks_yahoo extends Juno_Stocks {

    /**
     * Array of capabilities that this backend supports.
     * @var array $_capabilities
     */
    var $_capabilities = array('quotes' => true,
                               'charts' => true);

    /**
     * The base URL for Yahoo quotes.
     * @var string $_quoteUrl
     */
    var $_quoteUrl = 'http://quote.yahoo.com/d?f=snl1d1t1c1p2va2bapomwerr1dyj1x&s=';

    /**
     * The base URL for Yahoo charts.
     * @var string $_chartUrl
     */
    var $_chartUrl = 'http://chart.yahoo.com/c/';

    /**
     * The base URL for Yahoo intra-day charts.
     * @var string $_intraDayChartUrl
     */
    var $_intraDayChartUrl = 'http://ichart.yahoo.com/';

    function _getQuotes($symbol)
    {
        require_once 'Horde/Data.php';

        $url = $this->_quoteUrl . implode('+', $symbol);
        $data = Juno::_readUrl($url);
        if (is_a($data, 'PEAR_Error')) {
            Horde::logMessage($data->getMessage(), __FILE__, __LINE__, PEAR_LOG_ERR);
            return PEAR::raiseError(_("Could not fetch quote data."));
        }
        $temp = Horde::getTempFile('juno');
        $fp = fopen($temp, 'w');
        fwrite($fp, $data);
        fclose($fp);

        $csv = &Horde_Data::singleton('csv');
        $rows = $csv->importFile($temp, false, ',', '"', 22);

        $quotes = array();
        foreach ($rows as $row) {
            list($dayLow, $dayHigh) = explode('-', $row[13]);
            list($yearLow, $yearHigh) = explode('-', $row[14]);
            $quotes[String::lower($row[0])] = array(
                'symbol' => $row[0],
                'name' => $row[1],
                'lastPrice' => $row[2],
                'tradeDate' => $row[3] . ' ' . $row[4], 
                'changePrice' => $row[5], 
                'changePct' => $row[6],
                'volume' => $row[7],
                'avgvolume' => $row[8],
                'bestBid' => $row[9],
                'bestAsk' => $row[10], 
                'lastClose' => $row[11],
                'open' => $row[12],
                'high' => $dayHigh,
                'low' => $dayLow,
                'yearlyLow' => $yearLow,   
                'yearlyHigh' => $yearHigh,   
                'earnings' => $row[15],
                'peRatio' => $row[16],     
                'dividendDate' => $row[17],  
                'yieldPct' => $row[18], 
                'dividendAmt' => $row[19], 
                'marketCap' => $row[20],
                'exchange' => $row[21]);
        }

        return $quotes;
    }

    /**
     * Get a stock chart. Types available:
     *
     * Intraday - b = intraday/big; t = intraday/small;
     *        w = week/big; v = week/small
     * ex: $_intraDayChartUrl . '/b?s=aapl'
     *
     * Normal charts - 0b = year/small; 3m = 3month/big; 1y = 1yr/big;
     *         2y = 2yr/big; 5y = 5yr/big;
     *         add s to chart against S&P500
     *         add m to include moving average
     * ex: $_chartUrl . '0b/a/aapl.png'
     *
     * Dimensions are 512,288 for big charts and 192,96 for small
     * ones.
     */
    function _getChart($symbol, $type = 't', $modifier = '')
    {
        switch ($type) {
        case 'b':
        case 't':
        case 'w':
        case 'v':
            $url = $this->_intraDayChartUrl . $type . '?s=' . $symbol;
            break;

        case '0b':
        case '3m':
        case '1y':
        case '2y':
        case '5y':
            $url = $this->_chartUrl . $type;
            if ($modifier == 'm' || $modifier == 's') {
                $url .= $modifier;
            }
            $url .= '/' . substr($symbol, 0, 1) . '/' . $symbol . '.gif';
            break;
        }

        return Juno::_readUrl($url);
    }

}
